var util = require('util')
var assert = require('assert')

var async = require('async')
var _ = require('lodash')
var AWS = require('aws-sdk')

var ok = require('okay')

var options = {
  apiVersion: '2012-11-05'
}

var sqs = new AWS.SQS(options)

Queue.configure(options) {
  sqs = new AWS.SQS(options)
}

var Queue = function(name, url) {
  this.name = name
  this.url = url
  if(!this.name) {
    this.name = (url||'').split('/').pop()
  }
}

Queue.destroy = function(name, cb) {
  this._getUrl(ok(cb, function(url) {
    sqs.deleteQueue({QueueUrl: url}, cb)
  }))
}

Queue.create = function(name, options, cb) {
  if(typeof options == 'function') {
    cb = options
    options = {}
  }
  var options = {
    QueueName: name,
    Attributes: options
  }
  sqs.createQueue(options, ok(cb, function(res) {
    return cb(null, new Queue(null, res.QueueUrl))
  }))
}

Queue.list = function(prefix, cb) {
  var options = {
    QueueNamePrefix: prefix
  }
  sqs.listQueues(options, ok(cb, function(res) {
    cb(null, res.QueueUrls || [])
  }))
}

Queue.delete = function(url, cb) {
  sqs.deleteQueue({QueueUrl: url}, cb)
}

Queue.destroyAll = function(prefix, cb) {
  if(!prefix) {
    var msg =  'Sorry, you did not provide a prefix and I will not delete everything for you'
    cb(new Error(msg))
  }
  Queue.list(prefix, ok(cb, function(res) {
    async.forEach(res, Queue.delete, cb)
  }))
}

var urls = {}
Queue.prototype._getUrl = function(cb) {
  var self = this

  if(this.url) {
    urls[this.name] = this.url
    return cb(null, this.url)
  }

  if(urls[this.name]) {
    this.url = urls[this.name]
    return cb(null, this.url)
  }

  var options = {
    QueueName: this.name
  }

  sqs.getQueueUrl(options, ok(cb, function(res) {
    self.url = res.QueueUrl
    return self._getUrl(cb)
  }))
}

Queue.prototype.take = function(count, timeout, cb) {
  this._getUrl(ok(cb, function(url) {
    if(typeof timeout === 'function') {
      cb = timeout
      timeout = 1
    }
    var options = {
      QueueUrl: url,
      WaitTimeSeconds: timeout,
      MaxNumberOfMessages: count
    }
    sqs.receiveMessage(options, ok(cb, function(res) {
      var messages = []

      if(!res.Messages) {
        return cb(null, [])
      }

      for(var i = 0; i < res.Messages.length; i++) {
        var msg = res.Messages[i]
        messages.push({
          body: JSON.parse(msg.Body),
          receipt: msg.ReceiptHandle,
          id: msg.MessageId
        })
      }
      cb(null, messages)
    }))
  }))
}

Queue.prototype.pop = function(timeout, cb) {
  if(typeof timeout === 'function') {
    cb = timeout
    timeout = 1
  }
  this.take(1, timeout, ok(cb, function(res) {
    cb(null, res[0])
  }))
}

Queue.prototype.destroy = function(cb) {
  sqs.deleteQueue({QueueUrl: this.url}, cb)
}


var push = function(url, body, cb) {
  var options = {
    QueueUrl: url,
    MessageBody: body
  }
  if(typeof body != 'string') {
    body = JSON.stringify(body)
  }
  sqs.sendMessage(options, ok(cb, function(res) {
    cb(null, {id: res.MessageId})
  }))
}

Queue.prototype.push = function(body, cb) {
  var self = this
  this._getUrl(ok(cb, function(url) {
    if(util.isArray(body)) return self.pushAll(body, cb);
    return push(url, JSON.stringify(body), cb)
  }))
}

Queue.prototype.pushAll = function(items, cb) {
  var id = 0
  this._getUrl(ok(cb, function(url) {
    var messages = []
    for(var i = 0; i < items.length; i++) {
      var item = items[i]
      messages.push({
        Id: '' + id++,
        MessageBody: JSON.stringify(item)
      })
    }
    var options = {
      QueueUrl: url,
      Entries: messages
    }
    sqs.sendMessageBatch(options, ok(cb, function(res) {
      var result = {
        success: _.map(res.Successful, 'MessageId'),
        failure: _.map(res.Failed, function(msg) {
          return items[parseInt(msg.Id)]
        })
      }
      cb(null, result)
    }))
  }))
}

Queue.prototype.delete = function(item, cb) {
  this._getUrl(ok(cb, function(url) {
    var options = {
      QueueUrl: url,
      ReceiptHandle: item.receipt
    }
    sqs.deleteMessage(options, cb)
  }))
}

Queue.prototype.settings = function(cb) {
  this._getUrl(ok(cb, function(url) {
    var options = {
      QueueUrl: url,
      AttributeNames: ['All']
    }
    sqs.getQueueAttributes(options, ok(cb, function(res) {
      var result = {}
      var data = res.Attributes
      result.waiting = data.ApproximateNumberOfMessages;
      result.inProcess = data.ApproximateNumberOfMessagesNotVisible;
      result.messageTimeoutSeconds = data.VisibilityTimeout;
      result.retentionPeriodSeconds = data.MessageRetentionPeriod;
      cb(null, result)
    }))
  }))
}

module.exports = Queue
